@@ -4,23 +4,9 @@ syntactic sugar for Ajax requests in django
 
 from decorator import decorator
 
-try:
-    import simplejson as json
-except ImportError:
-    import json
+from django.http import Http404
 
-from django.http import HttpResponse
-
-
-class JsonResponse(HttpResponse):
-    """
-    HttpResponse descendant, which return response with ``application/json`` mimetype.
-    """
-    def __init__(self, data, status=200):
-        super(JsonResponse, self).__init__(
-            content=json.dumps(data),
-            mimetype='application/json', status=status
-        )
+from .http import JsonResponse
 
 
 def ajax(login_required=False, require_GET=False, require_POST=False,
@@ -32,6 +18,7 @@ def ajax(login_required=False, require_GET=False, require_POST=False,
     def my_ajax_view(request):
         return {'count': 42}
     """
+
     def ajax(f, request, *args, **kwargs):
         """ wrapper function """
         if login_required:
@@ -39,7 +26,7 @@ def ajax(login_required=False, require_GET=False, require_POST=False,
                 return JsonResponse({
                     'status': 'error',
                     'error': 'Unauthorized',
-                }, status=401)
+                    }, status=401)
 
         # check request method
         method = None
@@ -53,13 +40,34 @@ def ajax(login_required=False, require_GET=False, require_POST=False,
             return JsonResponse({
                 'status': 'error',
                 'error': 'Method not allowed',
-            }, status=405)
+                }, status=405)
+
+        try:
+            response = f(request, *args, **kwargs)
+        except Http404:
+            return JsonResponse({
+                'status': 'error',
+                'error': 'Not found',
+            }, status=404)
+
+        # check if it is an instance of HttpResponse
+        if hasattr(response, 'status_code'):
+            status_code = response.status_code
+            if status_code > 399:
+                return JsonResponse({
+                        'status': 'error',
+                        'error': response.content,
+                    },
+                    status=status_code
+                )
 
-        response = f(request, *args, **kwargs)
 
-        if isinstance(response, dict):
-            return JsonResponse(response)
-        else:
             return response
 
+        status_code = 200
+        if isinstance(response, tuple):
+            response, status_code = response
+
+        return JsonResponse(response, status=status_code)
+
     return decorator(ajax)
